@@ -1,4 +1,4 @@
-/* 
+/*
  * Gromit-MPX -- a program for painting on the screen
  *
  * Gromit Copyright (C) 2000 Simon Budig <Simon.Budig@unix-ag.org>
@@ -23,6 +23,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <lz4.h>
 
 #include "callbacks.h"
 #include "config.h"
@@ -32,15 +33,21 @@
 
 #include "paint_cursor.xpm"
 #include "erase_cursor.xpm"
+#include "coordlist_ops.h"
 
 
 
-GromitPaintContext *paint_context_new (GromitData *data, 
+GromitPaintContext *paint_context_new (GromitData *data,
 				       GromitPaintType type,
-				       GdkRGBA *paint_color, 
+				       GdkRGBA *paint_color,
 				       guint width,
 				       guint arrowsize,
                                        GromitArrowType arrowtype,
+                                       guint simpilfy,
+                                       guint radius,
+                                       guint maxangle,
+                                       guint minlen,
+                                       guint snapdist,
 				       guint minwidth,
 				       guint maxwidth)
 {
@@ -55,8 +62,12 @@ GromitPaintContext *paint_context_new (GromitData *data,
   context->minwidth = minwidth;
   context->maxwidth = maxwidth;
   context->paint_color = paint_color;
+  context->radius = radius;
+  context->maxangle = maxangle;
+  context->simplify = simpilfy;
+  context->minlen = minlen;
+  context->snapdist = snapdist;
 
-  
   context->paint_ctx = cairo_create (data->backbuffer);
 
   gdk_cairo_set_source_rgba(context->paint_ctx, paint_color);
@@ -65,7 +76,7 @@ GromitPaintContext *paint_context_new (GromitData *data,
   cairo_set_line_width(context->paint_ctx, width);
   cairo_set_line_cap(context->paint_ctx, CAIRO_LINE_CAP_ROUND);
   cairo_set_line_join(context->paint_ctx, CAIRO_LINE_JOIN_ROUND);
-  
+
   if (type == GROMIT_ERASER)
     cairo_set_operator(context->paint_ctx, CAIRO_OPERATOR_CLEAR);
   else
@@ -78,24 +89,28 @@ GromitPaintContext *paint_context_new (GromitData *data,
 }
 
 
-void paint_context_print (gchar *name, 
+void paint_context_print (gchar *name,
 			  GromitPaintContext *context)
 {
   g_printerr ("Tool name: \"%-20s\": ", name);
   switch (context->type)
   {
     case GROMIT_PEN:
-      g_printerr ("Pen,     "); break;
+      g_printerr ("Pen,        "); break;
     case GROMIT_LINE:
-      g_printerr ("Line,    "); break;
+      g_printerr ("Line,       "); break;
     case GROMIT_RECT:
-      g_printerr ("Rect,    "); break;
+      g_printerr ("Rect,       "); break;
+    case GROMIT_SMOOTH:
+      g_printerr ("Smooth,     "); break;
+    case GROMIT_ORTHOGONAL:
+      g_printerr ("Orthogonal, "); break;
     case GROMIT_ERASER:
-      g_printerr ("Eraser,  "); break;
+      g_printerr ("Eraser,     "); break;
     case GROMIT_RECOLOR:
-      g_printerr ("Recolor, "); break;
+      g_printerr ("Recolor,    "); break;
     default:
-      g_printerr ("UNKNOWN, "); break;
+      g_printerr ("UNKNOWN,    "); break;
   }
 
   g_printerr ("width: %u, ", context->width);
@@ -116,6 +131,17 @@ void paint_context_print (gchar *name,
         break;
       }
     }
+  if (context->type == GROMIT_SMOOTH || context->type == GROMIT_ORTHOGONAL)
+    {
+      g_printerr(" simplify: %u, ", context->simplify);
+      if (context->snapdist > 0)
+        g_printerr(" snap: %u, ", context->snapdist);
+    }
+  if (context->type == GROMIT_ORTHOGONAL)
+    {
+      g_printerr(" radius: %u, minlen: %u, maxangle: %u ",
+                 context->radius, context->minlen, context->maxangle);
+    }
   g_printerr ("color: %s\n", gdk_rgba_to_string(context->paint_color));
 }
 
@@ -134,9 +160,9 @@ void hide_window (GromitData *data)
       /* save grab state of each device */
       GHashTableIter it;
       gpointer value;
-      GromitDeviceData* devdata; 
+      GromitDeviceData* devdata;
       g_hash_table_iter_init (&it, data->devdatatable);
-      while (g_hash_table_iter_next (&it, NULL, &value)) 
+      while (g_hash_table_iter_next (&it, NULL, &value))
         {
           devdata = value;
           devdata->was_grabbed = devdata->is_grabbed;
@@ -144,7 +170,7 @@ void hide_window (GromitData *data)
       data->hidden = 1;
       release_grab (data, NULL); /* release all */
       gtk_widget_hide (data->win);
-      
+
       if(data->debug)
         g_printerr ("DEBUG: Hiding window.\n");
     }
@@ -161,9 +187,9 @@ void show_window (GromitData *data)
       /* restore grab state of each device */
       GHashTableIter it;
       gpointer value;
-      GromitDeviceData* devdata; 
+      GromitDeviceData* devdata;
       g_hash_table_iter_init (&it, data->devdatatable);
-      while (g_hash_table_iter_next (&it, NULL, &value)) 
+      while (g_hash_table_iter_next (&it, NULL, &value))
         {
           devdata = value;
           if(devdata->was_grabbed)
@@ -196,8 +222,8 @@ void clear_screen (GromitData *data)
   cairo_destroy(cr);
 
   GdkRectangle rect = {0, 0, data->width, data->height};
-  gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0); 
-  
+  gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0);
+
   if(!data->composited)
     {
       cairo_region_t* r = gdk_cairo_region_create_from_surface(data->backbuffer);
@@ -250,8 +276,8 @@ gint reshape (gpointer user_data)
 }
 
 
-void select_tool (GromitData *data, 
-		  GdkDevice *device, 
+void select_tool (GromitData *data,
+		  GdkDevice *device,
 		  GdkDevice *slave_device,
 		  guint state)
 {
@@ -265,7 +291,7 @@ void select_tool (GromitData *data,
 
   /* get the data for this device */
   GromitDeviceData *devdata = g_hash_table_lookup(data->devdatatable, device);
- 
+
   if (device)
     {
       slave_len = strlen (gdk_device_get_name(slave_device));
@@ -274,8 +300,8 @@ void select_tool (GromitData *data,
       name = (guchar*) g_strndup (gdk_device_get_name(device), len + 3);
       default_len = strlen(DEFAULT_DEVICE_NAME);
       default_name = (guchar*) g_strndup (DEFAULT_DEVICE_NAME, default_len + 3);
-      
-      
+
+
       /* Extract Button/Modifiers from state (see GdkModifierType) */
       req_buttons = (state >> 8) & 31;
 
@@ -349,7 +375,7 @@ void select_tool (GromitData *data,
                     devdata->cur_context = context;
                     success = 1;
                   }
-                
+
             }
           while (j<=3 && req_modifier >= (1u << j));
         }
@@ -374,7 +400,7 @@ void select_tool (GromitData *data,
 
   GdkCursor *cursor;
   if(devdata->cur_context && devdata->cur_context->type == GROMIT_ERASER)
-    cursor = data->erase_cursor; 
+    cursor = data->erase_cursor;
   else
     cursor = data->paint_cursor;
 
@@ -405,7 +431,8 @@ void snap_undo_state (GromitData *data)
   if(data->debug)
     g_printerr ("DEBUG: Snapping undo buffer %d.\n", data->undo_head);
 
-  copy_surface(data->undobuffer[data->undo_head], data->backbuffer);
+  undo_compress(data, data->backbuffer);
+  undo_temp_buffer_to_slot(data, data->undo_head);
 
   // Increment head position
   data->undo_head++;
@@ -431,23 +458,8 @@ void copy_surface (cairo_surface_t *dst, cairo_surface_t *src)
 }
 
 
-
-void swap_surfaces (cairo_surface_t *a, cairo_surface_t *b)
-{
-  int width = cairo_image_surface_get_width(a);
-  int height = cairo_image_surface_get_height(a);
-  cairo_surface_t *temp = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
-  copy_surface(temp, a);
-  copy_surface(a, b);
-  copy_surface(b, temp);
-  cairo_surface_destroy(temp);
-}
-
-
-
 void undo_drawing (GromitData *data)
 {
-  /* Swap undobuffer[head-1]->backbuffer */
   if(data->undo_depth <= 0)
     return;
   data->undo_depth--;
@@ -458,10 +470,12 @@ void undo_drawing (GromitData *data)
   if(data->undo_head < 0)
     data->undo_head += GROMIT_MAX_UNDO;
 
-  swap_surfaces(data->backbuffer, data->undobuffer[data->undo_head]);
+  undo_compress(data, data->backbuffer);
+  undo_decompress(data, data->undo_head, data->backbuffer);
+  undo_temp_buffer_to_slot(data, data->undo_head);
 
   GdkRectangle rect = {0, 0, data->width, data->height};
-  gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0); 
+  gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0);
 
   data->modified = 1;
 
@@ -476,27 +490,88 @@ void redo_drawing (GromitData *data)
   if(data->redo_depth <= 0)
     return;
 
-  swap_surfaces(data->backbuffer, data->undobuffer[data->undo_head]);
+  undo_compress(data, data->backbuffer);
+  undo_decompress(data, data->undo_head, data->backbuffer);
+  undo_temp_buffer_to_slot(data, data->undo_head);
 
   data->redo_depth--;
   data->undo_depth++;
   data->undo_head++;
   if(data->undo_head >= GROMIT_MAX_UNDO)
     data->undo_head -= GROMIT_MAX_UNDO;
-  
+
   GdkRectangle rect = {0, 0, data->width, data->height};
-  gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0); 
+  gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0);
 
   data->modified = 1;
-
+  
   if(data->debug)
     g_printerr("DEBUG: Redo drawing.\n");
 }
 
+/*
+ * compress image data and store it in undo_temp_buffer
+ *
+ * the undo_temp_buffer is successively grown in case it is too small
+ */
+void undo_compress(GromitData *data, cairo_surface_t *surface)
+{
+  char *raw_data = (char *)cairo_image_surface_get_data(surface);
+  guint bytes_per_row = cairo_image_surface_get_stride(surface);
+  guint rows = cairo_image_surface_get_height(surface);
+  size_t src_bytes = rows * bytes_per_row;
+
+  size_t dest_bytes;
+  for (;;)
+    {
+      dest_bytes = LZ4_compress_default(raw_data, data->undo_temp, src_bytes, data->undo_temp_size);
+      if (dest_bytes == 0)
+        {
+          data->undo_temp_size *= 2;
+          data->undo_temp = g_realloc(data->undo_temp, data->undo_temp_size);
+        }
+      else
+        {
+          data->undo_temp_used = dest_bytes;
+          break;
+        }
+    }
+}
 
 
+/*
+ * copy undo_temp to undo slot, growing undo buffer if required
+ */
+void undo_temp_buffer_to_slot(GromitData *data, gint undo_slot)
+{
+    const size_t required = data->undo_temp_used;
+    if (data->undo_buffer_size[undo_slot] < required)
+      {
+        data->undo_buffer[undo_slot] = g_realloc(data->undo_buffer[undo_slot], required);
+        data->undo_buffer_size[undo_slot] = required;
+      }
+    data->undo_buffer_used[undo_slot] = required;
+    memcpy(data->undo_buffer[undo_slot], data->undo_temp, required);
+}
 
+/*
+ * decompress undo slot data and store it in cairo surface
+ */
+void undo_decompress(GromitData *data, gint undo_slot, cairo_surface_t *surface)
+{
+  char *dest_data = (char *)cairo_image_surface_get_data(surface);
+  guint bytes_per_row = cairo_image_surface_get_stride(surface);
+  guint rows = cairo_image_surface_get_height(surface);
+  size_t dest_bytes = rows * bytes_per_row;
 
+  char *src_data = data->undo_buffer[undo_slot];
+  size_t src_bytes = data->undo_buffer_used[undo_slot];
+
+  if (LZ4_decompress_safe(src_data, dest_data, src_bytes, dest_bytes) < 0) {
+    g_printerr("Fatal error occurred decompressing image data\n");
+    exit(1);
+  }
+}
 
 /*
  * Functions for handling various (GTK+)-Events
@@ -575,7 +650,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   gdk_rgba_parse(data->red, "#FF0000");
 
 
-  /* 
+  /*
      CURSORS
   */
   GdkPixbuf* paint_cursor_pixbuf = gdk_pixbuf_new_from_xpm_data(paint_cursor_xpm);
@@ -599,24 +674,26 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   /* SHAPE SURFACE*/
   cairo_surface_destroy(data->backbuffer);
   data->backbuffer = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, data->width, data->height);
-  
+
+  // original state for LINE and RECT tool
+  cairo_surface_destroy(data->aux_backbuffer);
+  data->aux_backbuffer = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, data->width, data->height);
+
   /*
     UNDO STATE
   */
   data->undo_head = 0;
   data->undo_depth = 0;
   data->redo_depth = 0;
-  int i;
-  for (i = 0; i < GROMIT_MAX_UNDO; i++)
+  data->undo_temp_size = 0x10000;
+  data->undo_temp = g_malloc(data->undo_temp_size);
+  data->undo_temp_used = 0;
+  for (int i = 0; i < GROMIT_MAX_UNDO; i++)
     {
-      cairo_surface_destroy(data->undobuffer[i]);
-      data->undobuffer[i] = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, data->width, data->height);
+      data->undo_buffer_size[i] = 0;
+      data->undo_buffer[i] = NULL;
     }
 
-  // original state for LINE and RECT tool
-  cairo_surface_destroy(data->aux_backbuffer);
-  data->aux_backbuffer = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, data->width, data->height);
-
   /* EVENTS */
   gtk_widget_add_events (data->win, GROMIT_WINDOW_EVENTS);
   g_signal_connect (data->win, "draw",
@@ -635,15 +712,15 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
                     G_CALLBACK (on_device_removed), data);
   g_signal_connect (data->win, "motion_notify_event",
 		    G_CALLBACK (on_motion), data);
-  g_signal_connect (data->win, "button_press_event", 
+  g_signal_connect (data->win, "button_press_event",
 		    G_CALLBACK (on_buttonpress), data);
   g_signal_connect (data->win, "button_release_event",
 		    G_CALLBACK (on_buttonrelease), data);
   /* disconnect previously defined selection handlers */
-  g_signal_handlers_disconnect_by_func (data->win, 
+  g_signal_handlers_disconnect_by_func (data->win,
 					G_CALLBACK (on_clientapp_selection_get),
 					data);
-  g_signal_handlers_disconnect_by_func (data->win, 
+  g_signal_handlers_disconnect_by_func (data->win,
 					G_CALLBACK (on_clientapp_selection_received),
 					data);
   /* and re-connect them to mainapp functions */
@@ -659,7 +736,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
       gtk_widget_shape_combine_region(data->win, r);
       cairo_region_destroy(r);
     }
-  
+
 
   /* reset settings from client setup */
   gtk_selection_remove_all (data->win);
@@ -678,7 +755,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   gtk_selection_add_target (data->win, GA_CONTROL, GA_LINE, 10);
 
 
- 
+
 
   /*
    * Parse Config file
@@ -700,8 +777,8 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   // might have been in key file
   gtk_widget_set_opacity(data->win, data->opacity);
 
-  /* 
-     FIND HOTKEY KEYCODE 
+  /*
+     FIND HOTKEY KEYCODE
   */
   if (data->hot_keyval)
     {
@@ -729,7 +806,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
     }
 
   /*
-     FIND UNDOKEY KEYCODE 
+     FIND UNDOKEY KEYCODE
   */
   if (data->undo_keyval)
     {
@@ -757,7 +834,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
     }
 
 
-  /* 
+  /*
      INPUT DEVICES
   */
   data->devdatatable = g_hash_table_new(NULL, NULL);
@@ -771,15 +848,15 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   hide_window (data);
 
   data->timeout_id = g_timeout_add (20, reshape, data);
- 
+
   data->modified = 0;
 
   data->default_pen =
-    paint_context_new (data, GROMIT_PEN, data->red, 7,
-                       0, GROMIT_ARROW_END, 1, G_MAXUINT);
+    paint_context_new (data, GROMIT_PEN, data->red, 7, 0, GROMIT_ARROW_END,
+                       5, 10, 15, 25, 0, 1, G_MAXUINT);
   data->default_eraser =
-    paint_context_new (data, GROMIT_ERASER, data->red, 75,
-                       0, GROMIT_ARROW_END, 1, G_MAXUINT);
+    paint_context_new (data, GROMIT_ERASER, data->red, 75, 0, GROMIT_ARROW_END,
+                       5, 10, 15, 25, 0, 1, G_MAXUINT);
 
   gdk_event_handler_set ((GdkEventFunc) main_do_event, data, NULL);
   gtk_key_snooper_install (snoop_key_press, data);
@@ -787,7 +864,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   if (activate)
     acquire_grab (data, NULL); /* grab all */
 
-  /* 
+  /*
      TRAY ICON
   */
   data->trayicon = app_indicator_new (PACKAGE_NAME,
@@ -820,6 +897,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
 
   GtkWidget* sep1_item = gtk_separator_menu_item_new();
   GtkWidget* intro_item = gtk_menu_item_new_with_mnemonic(_("_Introduction"));
+  GtkWidget* edit_config_item = gtk_menu_item_new_with_mnemonic(_("_Edit Config"));
   GtkWidget* issues_item = gtk_menu_item_new_with_mnemonic(_("_Report Bug / Request Feature"));
   GtkWidget* support_item = gtk_menu_item_new_with_mnemonic(_("_Support Gromit-MPX"));
   GtkWidget* about_item = gtk_menu_item_new_with_mnemonic(_("_About"));
@@ -842,6 +920,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
 
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), sep1_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), intro_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), edit_config_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), issues_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), support_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), about_item);
@@ -904,6 +983,9 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
   g_signal_connect(G_OBJECT (intro_item), "activate",
 		   G_CALLBACK (on_intro),
 		   data);
+  g_signal_connect(G_OBJECT (edit_config_item), "activate",
+		   G_CALLBACK (on_edit_config),
+		   data);
   g_signal_connect(G_OBJECT (issues_item), "activate",
 		   G_CALLBACK (on_issues),
 		   data);
@@ -928,6 +1010,7 @@ void setup_main_app (GromitData *data, int argc, char ** argv)
 
   gtk_widget_show (sep1_item);
   gtk_widget_show (intro_item);
+  gtk_widget_show (edit_config_item);
   gtk_widget_show (issues_item);
   gtk_widget_show (support_item);
   gtk_widget_show (about_item);
@@ -1016,9 +1099,9 @@ int main_client (int argc, char **argv, GromitData *data)
          {
            if (argc - (i+1) == 6) /* this command must have exactly 6 params */
              {
-              
+
                 // check if provided values are valid coords on the screen
-               if (atoi(argv[i+1]) < 0 || atoi(argv[i+1]) > (int)data->width || 
+               if (atoi(argv[i+1]) < 0 || atoi(argv[i+1]) > (int)data->width ||
                    atoi(argv[i+2]) < 0 || atoi(argv[i+2]) > (int)data->height ||
                    atoi(argv[i+3]) < 0 || atoi(argv[i+3]) > (int)data->width ||
                    atoi(argv[i+4]) < 0 || atoi(argv[i+4]) > (int)data->height)
@@ -1031,15 +1114,15 @@ int main_client (int argc, char **argv, GromitData *data)
                       g_printerr ("Thickness must be atleast 1\n");
                       wrong_arg = TRUE;
                     }
-               else 
+               else
                     {
                       data->clientdata = g_strjoin(" ", argv[i+1], argv[i+2], argv[i+3], argv[i+4], argv[i+5], argv[i+6], NULL);
                     }
-              
+
                action = GA_LINE;
                i += 6;
              }
-           else 
+           else
              {
                g_printerr ("-l requires 6 parameters: startX, startY, endX, endY, color, thickness\n");
                wrong_arg = TRUE;
